/*  $Id: AbstractCommand.java 1066 2010-07-07 16:47:36Z sdevaux $
 *
 * Tabs = 4 chars
 * copyright (C) 2010 Sebastien Devaux
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 *
 *              Sebastien Devaux <sebastien.devaux@laposte.net>
 */
package net.eduvax.util;

import java.util.Hashtable;
import java.util.StringTokenizer;

/**
 * Default command.
 * Just have to implement a run method.
 * @version $Id: AbstractCommand.java 1066 2010-07-07 16:47:36Z sdevaux $
 * @author Sebastien Devaux
 */
public abstract class AbstractCommand implements Command {
    /**
     * Create new command
     */
    protected AbstractCommand() {
        _params=new Hashtable<String,String>();
    }
    /** 
     * Default command parameter settings.
     * All parameter are set with one string using following format :
     * <code>param1=val1&param2=val2&...&paramn=valn</code>
     */
    public void setParameters(String param) {
        _queryString=param;
        StringTokenizer st=new StringTokenizer(param,"&");
        while (st.hasMoreTokens()) {
            StringTokenizer stp=new StringTokenizer(st.nextToken(),"=");
            String name=stp.nextToken();
            String value="";
            if (stp.hasMoreTokens()) {
                value=stp.nextToken();
            }
            setParameter(name,value);
        }
    }
    /** 
     * Set one parameter.
     * @param name parameter name
     * @param value parameter value
     */
    public void setParameter(String name,String value) {
        _params.put(name,value);
    }
    /**
     * get one paramater
     * @param name requested paramter name
     * @return found parameter value, null when not found.
     */
    public String getParameter(String name) {
		return _params.get(name);
    }
    /**
     * get query strings.
     * get all params as a query string 
     * (like "param1=val1&amp;param2=val2&amp;...).
     * In facts returns only the string defined with setParameters(String). 
     * Individualy set parameters using setParameter(String,String) are not
     * included in returned query string.
     * @return query string, or null if none.
     */
    protected String getQueryString() {
        return _queryString;
    }
    /** Store all param as query string when all params set at once. */
    private String _queryString;
    /** Store params */
    private Hashtable<String,String> _params;
    /** Error handler*/
    private ErrorHandler _errHandler=null;
    /**
     * Change error handler
     * @param handler new handler
     */
    public void setErrorHandler(ErrorHandler handler) {
        _errHandler=handler;
    }
    /**
     * @return error handler, null when no handler set.
     */
    protected ErrorHandler getErrorHandler() {
        return _errHandler;
    }
}
