/*  $Id: Error.java 1335 2012-08-25 13:27:49Z sdevaux $
 *
 * Tabs = 4 chars
 * copyright (C) 2002-2012 Sebastien Devaux
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 *
 *              Sebastien Devaux <sebastien.devaux@laposte.net>
 */
package net.eduvax.util;

import java.util.Iterator;
import java.util.MissingFormatArgumentException;
import java.util.MissingResourceException;
import java.util.ResourceBundle;
import java.util.Vector;

/**
 * Error descriptor.
 * Retrieves message from a package ressource bundle named as 
 * the name of the class.
 * All package that need to have its own ressource bundle should
 * create a subclass, at least to get bundle loaded from file named
 * as the class.
 * TODO: Add date field ?
 */
public class Error {
    /**
     * Create new error.
     * ident specifies the error message.
     * @param ident error ident (i.e error number or equivalent). Several instance
     * with same ident are allowed.
     * @param l error level
     */
    public Error(String ident,ELevel l) {
        _id=ident;
        _level=l;
    }
    public Error(String ident,ELevel l,Exception exception) {
        _id=ident;
        _level=l;
        _ex=exception;
    }
    /**
     * Retrieve localized error message about error.
     */ 
    public String getMessage() {
        Exception ex=null;
        ResourceBundle resMessages=App.get().getResource(getClass().getName());
        String msg="";
        try {
            msg=resMessages.getString(_id);
            // replace %s with infos
            msg=String.format(msg,_args.toArray())+" [Err: "+_id+"]";
        }
        catch (MissingResourceException mrex) {
            ex=mrex;
            msg="[Err: "+_id+"]";
        }
        catch (MissingFormatArgumentException mfaex) {
            ex=mfaex;
        }
        if (ex!=null) {
            StringBuffer sb=new StringBuffer();
            sb.append(msg);
            sb.append("[args:");
            Iterator<Object> it=_args.iterator();
            while (it.hasNext()) {
                sb.append(" "+it.next());
            }
            sb.append("]");
            msg=sb.toString();
        }    
        return msg;
    }
    /**
     * Add argument to error.
     * anything useable as information complement about error.
     * May be use in message construction.
     * Error messages may includes % fields (as printf, see String.format() ). Arguments
     * are used to format message in order of addition.
     * @param arg error argument to add
     */
    public void addArg(Object arg) {
        _args.add(arg);
    }
    /** Error id */
    private String _id;
    /** Exception related to error */
    private Exception _ex=null;
    public Exception getException() {
        return _ex;
    }
    /** Error level */
    private ELevel _level;
    /**
     * Get error level.
     * @return one of L_ constants.
     */ 
    public ELevel getLevel() {
        return _level;
    }
    /** Error args : complementary information about error */
    private Vector<Object> _args=new Vector<Object>();
    /** Error levels */ 
    public enum ELevel { UNDEF, WARN, ERR, FATAL };
}
