/*  $Id: WeakHashtable.java 1035 2010-06-15 13:08:33Z sdevaux $
 *
 * Tabs = 4 chars
 * copyright (C) 2002-2012 Sebastien Devaux
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 *
 *              Sebastien Devaux <sebastien.devaux@laposte.net>
 */
package net.eduvax.xml;

import java.io.FileOutputStream;
import java.io.IOException;
import org.xml.sax.SAXException;

import net.eduvax.xml.SaxWriter;
import net.eduvax.util.App;
import net.eduvax.util.ErrorHandler;

/** Service de base pour l'écriture de flux XML. */
public abstract class AXmlWriter implements Runnable {
    /** 
     * Interface d'écriture XML
     */ 
    private SaxWriter _xml=null;
    /** Donne accès au writer XMl sous-jacent pour les sous-classes.
     * @return 
     * - writer XML.
     * - null si Run non commencé.
     */ 
    protected SaxWriter getSaxWriter() {
        return _xml;
    }
    /** 
     * Gestionnaire d'erreur
     * @djin.relation aggregation.
     */ 
    private ErrorHandler _errHandler;
    /** Chemin du fichier cible */
    private String _path;

    /** Créer nouveau XmlWriter 
     * @param path Chemin du fichier cible.
     */ 
    public AXmlWriter(String path) {
        _path=path;
        _errHandler=App.get().getErrHandler();
    }

    /** Definition gestionnaire d'erreur. 
     * @param handler Gestionnaire d'erreur.
     */ 
    public void setErrHandler(ErrorHandler handler) {
        _errHandler=handler;
    }
    /** Méthode à surcharger pour définir les actions d'écriture.
     *  */
    public abstract void oRun() throws SAXException;
    /** Mise en oeuvre Runnable. 
     * - Initialisation écriture.
     * - Ecriture prologue document XML.
     * - Délégation écritures à ORun.
     * - Fermeture du flux.
     */  
    public void run() {
        FileOutputStream out=null;
        try {
            out=new FileOutputStream(_path);
            _xml=new SaxWriter(out);
            oRun();
        }
        catch (IOException ioex) {
            Error err=new Error(
                ""+getClass()+".ioex",Error.ELevel.ERR,ioex);
            err.addArg(_path);
            _errHandler.handle(err);
        }
        catch (SAXException saxex) {
            Error err=new Error(
                ""+getClass()+".saxex",Error.ELevel.ERR,saxex);
            err.addArg(_path);
            _errHandler.handle(err);
        }
        try {
            _xml.flush();
            out.close();
        }
        catch (Exception ex) {
            Error err=new Error(
                ""+getClass()+".close",Error.ELevel.ERR,ex);
            err.addArg(_path);
            _errHandler.handle(err);
        }
    }
}
